from pkg_resources import get_distribution, DistributionNotFound

try:
    __version__ = get_distribution(__name__).version
except DistributionNotFound:
    # package is not installed
    __version__ = "not_intalled"
    pass



import asyncio
from panoramisk import Manager
from pprint import pprint
from prometheus_client import start_http_server
from prometheus_client import Gauge, Counter
from asterisk_exporter.log import log


def start_server(host, port, user, password):
    manager = Manager(loop=asyncio.get_event_loop(), host="localhost", username=user, secret=password)

    @manager.register_event('FullyBooted')
    def on_asterisk_FullyBooted(manager, msg):
        log.debug(msg)
        resp_iax_peer_list = yield from manager.send_action({'Action': 'IAXpeerlist'})
        try:
            iax_peers = [ 'IAX2/' + msg.ObjectName for msg in resp_iax_peer_list if msg.Event=='PeerEntry' and 'OK' in msg.Status ]
        except AttributeError:
            iax_peers = []
        for iax_peer in iax_peers:
            peer_status_gauge.labels(proto='IAX2', peer=iax_peer).set(1)
        resp_pjsip_contacts = yield from manager.send_action({'Action': 'PJSIPShowContacts'})
        try:
            pjsip_peers = [ 'PJSIP/' + msg.Endpoint for msg in resp_pjsip_contacts if msg.Event=='ContactList' and msg.Status == 'Reachable' ]
        except AttributeError:
            pjsip_peers = []
        for pjsip_peer in pjsip_peers:
            peer_status_gauge.labels(proto='SIP', peer=pjsip_peer).set(1)
        resp_sip_peer_list = yield from manager.send_action({'Action': 'SIPPeers'})
        try:
            sip_peers = [ 'SIP/' + msg.ObjectName for msg in resp_sip_peer_list if msg.Event=='PeerEntry' and 'OK' in msg.Status ]
        except AttributeError:
            sip_peers = []
        for sip_peer in sip_peers:
            peer_status_gauge.labels(proto='SIP', peer=sip_peer).set(1)

    @manager.register_event('PeerStatus')
    def on_peer_status(manager, msg):
        log.debug(msg)
        if msg.ChannelType == 'PJSIP':
            proto = 'SIP'
        else:
            proto = msg.ChannelType
        if msg.PeerStatus == 'Unreachable' or msg.PeerStatus == 'Unregistered':
            peer_status_gauge.labels(proto=proto, peer=msg.Peer).set(0)
        if msg.PeerStatus == 'Reachable' or msg.PeerStatus == 'Registered':
            peer_status_gauge.labels(proto=proto, peer=msg.Peer).set(1)

    @manager.register_event('Newchannel')
    def on_channel_creation(manager, msg):
        log.debug(msg)
        channel_type = msg.Channel.split('/')[0]
        current_channels_gauge.labels(type=channel_type).inc()
        channels_counter.labels(type=channel_type).inc()

    @manager.register_event('Hangup')
    def on_channel_deletion(manager, msg):
        log.debug(msg)
        channel_type = msg.Channel.split('/')[0]
        current_channels_gauge.labels(type=channel_type).dec()

    start_http_server(port, host)
    peer_status_gauge = Gauge('asterisk_peer_status',
                              'Status of iax and sip peers',
                              ['proto', 'peer'])
    current_channels_gauge = Gauge('asterisk_current_channels',
                                  'Number of active channels',
                                  ['channel_type'])
    channels_counter = Counter('asterisk_channels_total',
                               'Total number of created channels',
                               ['channel_type'])
    current_channels_gauge.labels('IAX2').set(0)
    current_channels_gauge.labels('PJSIP').set(0)
    current_channels_gauge.labels('SIP').set(0)
    channels_counter.labels('IAX2')
    channels_counter.labels('PJSIP')
    channels_counter.labels('SIP')
    manager.connect()
    manager.loop.run_forever()



import click
import sentry_sdk
from asterisk_exporter import __version__
from sentry_sdk.integrations.logging import LoggingIntegration
import asterisk_exporter.exporter as exporter
import logging
import os

sentry_dsn =  os.environ.get('SENTRY_DSN')
if sentry_dsn:
    sentry_logging = LoggingIntegration(
        level=logging.INFO,
        event_level=logging.CRITICAL
    )

    sentry_sdk.init(
        sentry_dsn,
        traces_sample_rate=1.0,
        release="asterisk_exporter@{}".format(__version__),
        integrations=[sentry_logging]
    )

@click.group(help='')
def cli():
    pass

@click.command()
@click.option('-h', '--host', required= True, type=str, help= "address on which to bind")
@click.option('-p', '--port', required= True, type=int, help= "port on which to bind")
@click.option('-u', '--user', required= True, type=str, help= "asterisk manager user")
@click.option('-w', '--password', required= True, type=str, help= "asterisk manager password")
def start(host, port, user, password):
    exporter.start_server(host, port, user, password)

cli.add_command(start)

if __name__ == '__main__':
    cli()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import io
import os
import sys
from shutil import rmtree

from setuptools import find_packages, setup, Command

# Package meta-data.
NAME = 'asterisk_exporter'
DESCRIPTION = 'Prometheus exporter for Asterisk.'
URL = 'https://github.com/sensorberg/asterisk-exporter'
EMAIL = 'infrastucture@sensorberg.com'
AUTHOR = 'sensorberg-infra'
REQUIRES_PYTHON = '>=3.5'

# What packages are required for this module to be executed?
with open('requirements.txt') as f:
    REQUIRED = f.read().splitlines()

# What packages are optional?
EXTRAS = {
    # 'fancy feature': ['django'],
}

# The rest you shouldn't have to touch too much :)
# ------------------------------------------------
# Except, perhaps the License and Trove Classifiers!
# If you do change the License, remember to change the Trove Classifier for that!

here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!
try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

about = {}

# Where the magic happens:
setup(
    name=NAME,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    use_scm_version=True,
    setup_requires=['setuptools_scm'],
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    # If your package is a single module, use this instead of 'packages':
    # py_modules=['mypackage'],

    entry_points={
       'console_scripts': ['asterisk_exporter=asterisk_exporter.app:cli',]
    },
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
    license='GPL-3.0',
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ],
    # $ setup.py publish support.
    cmdclass={
    },
)
